package org.deephacks.tools4j.config.internal.core;

import org.deephacks.tools4j.config.Config;
import org.deephacks.tools4j.config.ConfigDefault;
import org.deephacks.tools4j.config.RuntimeContext;
import org.deephacks.tools4j.config.model.AbortRuntimeException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.event.Observes;
import javax.enterprise.inject.spi.*;
import javax.inject.Singleton;
import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@Singleton
public class ConfigCdiBootstrap implements Extension {
    private static final Logger log = LoggerFactory.getLogger(ConfigCdiBootstrap.class);
    private static final Set<Class<?>> schemas = new HashSet<>();
    private static final Set<Object> defaults = new HashSet<>();

    public void afterBeanDiscovery(@Observes AfterBeanDiscovery event, BeanManager bm) {
        event.addContext(new ConfigCdiContext(bm));
    }

    public void afterDeploymentValidation(@Observes AfterDeploymentValidation event, BeanManager bm) {
        RuntimeContext ctx = getInstance(RuntimeContext.class, bm);
        for (Class<?> cls : schemas){
            try {
                ctx.register(cls);
            } catch (AbortRuntimeException e) {
                if(e.getEvent().getCode() < 200){
                    log.warn(e.getMessage());
                }
            }
        }
        ctx.registerDefault(defaults.toArray(new Object[0]));
    }

    public static <T> T getInstance(Class<T> cls, BeanManager bm){
        Set<Bean<?>> beans = bm.getBeans(cls);
        Bean<?> bean = bm.resolve(beans);
        CreationalContext cc = bm.createCreationalContext(bean);
        return (T) bm.getReference(bean, cls, cc);
    }

    public <X> void processAnnotatedType(@Observes ProcessAnnotatedType<X> pat) {
        AnnotatedType<?> type = pat.getAnnotatedType();
        if (type.isAnnotationPresent(Config.class)) {
            schemas.add(pat.getAnnotatedType().getJavaClass());
        }
        for (AnnotatedMethod<?> m : type.getMethods()) {
            ConfigDefault def = m.getAnnotation(ConfigDefault.class);
            if (def != null && m.isStatic()) {
                Method method = m.getJavaMember();
                try {
                    Object o = method.invoke(null, (Object[]) null);
                    if (o instanceof List) {
                        List<?> list = (List<?>) o;
                        for (Object obj : list) {
                            defaults.add(obj);
                        }
                    } else {
                        defaults.add(o);
                    }
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }
        }
    }

}
